/*
 * Copyright 2017 NXP
 * All rights reserved.
 *
 * THIS SOFTWARE IS PROVIDED BY NXP "AS IS" AND ANY EXPRESSED OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL NXP OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
 * IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <stddef.h>
#include "adc_pal.h"
#include "adc_irq.h"
#include "device_registers.h"
#include "interrupt_manager.h"
#include "osif.h"


/*******************************************************************************
 * Definitions
 ******************************************************************************/


/*! @cond DRIVER_INTERNAL_USE_ONLY */
typedef struct
{
    uint16_t bufferLength;            /*!< Length of the buffer associated with the current active conversion group */
    uint16_t currentBufferOffset;     /*!< Offset (in elements) of the next position to be written in the result buffer */
    bool notificationEn;              /*!< Flag for enabling/disabling notification */
} adc_group_state_t;

/*!
 * @brief Runtime state of the ADC PAL
 *
 * This structure is used by the ADC PAL for storing internal information, needed at runtime.
 */
typedef struct
{
    const adc_group_config_t * groupArray;              /*!< Pointer to the array of group configurations */
    uint16_t numGroups;                                 /*!< Number of group configurations available in the input array */
    uint32_t activeGroupIdx;                            /*!< Index of the active group (HW trigger group enabled or executing, or SW triggered group executing) */
    adc_group_state_t activeGroupState;                 /*!< State of the active group (HW trigger group enabled or executing, or SW triggered group executing) */
    bool activeGroupFlag;                               /*!< True/False - group conversion active/not active */
} adc_pal_state_t;

#if defined(ADC_PAL_S32K1xx)

#define ADC_PAL_PDB_CHAN            (0u)                /*!< PDB channel used for triggering ADC */
#define ADC_PAL_TRGMUX_IDX          (0u)                /*!< TRGMUX instance used by ADC PAL */
#define ADC_PAL_MAX_CONVS_IN_GROUP  (PDB_DLY_COUNT)     /*!< Maximum number of ADC conversions in a group of conversions. */

const trgmux_target_module_t adcPalTrgmuxTarget[NUMBER_OF_ADC_PAL_INSTANCES] = {
#if (NUMBER_OF_ADC_PAL_INSTANCES >= 1u)
    TRGMUX_TARGET_MODULE_PDB0_TRG_IN,
#endif
#if (NUMBER_OF_ADC_PAL_INSTANCES >= 2u)
    TRGMUX_TARGET_MODULE_PDB1_TRG_IN
#endif
};

static inline void ADC_ConfigPdbAndPretriggers(const uint32_t instance, const pdb_trigger_src_t trgSrc,  const adc_group_config_t * currentGroupCfg);
static void ADC_ConfigGroup(const uint32_t instance, const uint32_t groupIdx, const bool hwTriggerFlag);
static status_t ADC_StopGroupBlocking(const uint32_t instance, const uint32_t timeout);

#elif defined(ADC_PAL_MPC574xC_G_R)

#define ADC_PAL_MAX_CONVS_IN_GROUP_HW_TRIG  (BCTU_LISTCHR__COUNT * 2u) /*!< Maximum number of ADC conversions in a hw triggered group of conversions - each LISTCHR reg contains 2 list elements */

static inline uint8_t ADC_GetChanIdx(const adc_input_chan_t adcPalInputChan);
static inline uint8_t ADC_GetAdcIdx(const adc_input_chan_t adcPalInputChan);
static uint16_t ADC_GetUsedAdcInstances(const adc_pal_instance_t instance);
static status_t ADC_ConfigHwTriggeredGroup(const adc_pal_instance_t instance, const adc_group_config_t * const groupConfig, uint8_t bctuStartListIdx);

#endif /* defined(ADC_PAL_MPC574xC_G_R) */


#if (defined(ADC_PAL_S32K1xx) || defined(ADC_PAL_MPC574xC_G_R))

static void ADC_PalStateUpdateStart(const adc_pal_instance_t instance, const uint32_t groupIdx);
static void ADC_CallNotificationCb(const adc_pal_state_t * const palState);

#endif /* (defined(ADC_PAL_S32K1xx) || defined(ADC_PAL_MPC574xC_G_R)) */
/*! @endcond */

/*******************************************************************************
 * Variables
 ******************************************************************************/
/* Static variable to store the PAL state information */
static adc_pal_state_t adcPalState[NUMBER_OF_ADC_PAL_INSTANCES];

/*******************************************************************************
 * Public Functions
 ******************************************************************************/


/*FUNCTION**********************************************************************
 *
 * Function Name : ADC_Init
 * Description   : This function initializes the ADC PAL instance, including the
 * other platform specific HW units used together with ADC.
 *
 * Implements : ADC_Init_Activity
 *END**************************************************************************/
status_t ADC_Init(const adc_pal_instance_t instance, const adc_config_t * const config)
{
    DEV_ASSERT(instance < NUMBER_OF_ADC_PAL_INSTANCES);
    DEV_ASSERT(config != NULL);
    DEV_ASSERT(config->extension != NULL);
    DEV_ASSERT(config->groupConfigArray != NULL);

    status_t status = STATUS_SUCCESS;
    uint16_t idx = 0;
    (void) idx;

    /* Initialize PAL state structure */
    adcPalState[instance].groupArray       = config->groupConfigArray;
    adcPalState[instance].activeGroupIdx   = 0u;
    adcPalState[instance].numGroups        = config->numGroups;
    adcPalState[instance].activeGroupFlag  = false;

#if defined(ADC_PAL_S32K1xx)
    extension_adc_s32k1xx_t * extension = (extension_adc_s32k1xx_t *)(config->extension);
    adc_converter_config_t adcCfg;

    DEV_ASSERT(extension->inputClock <= NUMBER_OF_ALT_CLOCKS);

#if defined (CUSTOM_DEVASSERT) || defined (DEV_ERROR_DETECT)
    {
        /* Iterate over all conversion groups and DEV_ASSERT if number of conversions is larger than max supported */
        for(idx = 0u; idx < adcPalState[instance].numGroups; idx++)
        {
            DEV_ASSERT(adcPalState[instance].groupArray[idx].numChannels <= ADC_PAL_MAX_CONVS_IN_GROUP);
        }
    }
#endif /* defined (CUSTOM_DEVASSERT) || defined (DEV_ERROR_DETECT) */

    /* ADC configuration */
    ADC_DRV_Reset(instance);

    ADC_DRV_InitConverterStruct(&adcCfg);
    adcCfg.trigger                = ADC_TRIGGER_HARDWARE; /* ADC in hw trigger mode - sw triggering shall be done via PDB */
    adcCfg.sampleTime             = config->sampleTicks;
    adcCfg.clockDivide            = extension->clockDivide;
    adcCfg.resolution             = extension->resolution;
    adcCfg.inputClock             = extension->inputClock;
    adcCfg.voltageRef             = extension->voltageRef;
    adcCfg.supplyMonitoringEnable = extension->supplyMonitoringEnable;
    adcCfg.pretriggerSel          = ADC_PRETRIGGER_SEL_PDB; /* configure pretriggers 0->3 to be routed from PDB */

    ADC_DRV_ConfigConverter(instance, &adcCfg);

    ADC_DRV_AutoCalibration(instance);

    /* PDB init shall only be called from StartConversion() & EnableHardwareTrigger()
     * because PDB input source and continuous conversion enable need to be configured for each call. */

    /* Only reset the trgmux target register corresponding to PDB instance.
     * Calling TRGMUX init would reset all TRGMUX target registers - affecting other modules. */
    status = TRGMUX_DRV_SetTrigSourceForTargetModule(ADC_PAL_TRGMUX_IDX, TRGMUX_TRIG_SOURCE_DISABLED, adcPalTrgmuxTarget[instance]);
    if(status != STATUS_SUCCESS)
    {
        status = STATUS_ERROR;
    }

#elif defined(ADC_PAL_MPC574xC_G_R)

    const extension_adc_mpc574xc_g_r_t * const extension = (extension_adc_mpc574xc_g_r_t *)(config->extension);
    const adc_group_config_t * currentGroup;
    adc_conv_config_t adcCfg;
    bctu_config_t bctuCfg;
    status_t bctuStatus;

    ADC_DRV_GetDefaultConfigConverter(&adcCfg);
    adcCfg.sampleTimeExternal   = config->sampleTicks;
    adcCfg.sampleTimeInternal   = config->sampleTicks;
    adcCfg.sampleTimePrecision  = config->sampleTicks;
    adcCfg.clkSelect    = extension->clkSelect;
    adcCfg.refSelect    = extension->refSelect;
    adcCfg.autoClockOff = extension->autoClockOff;
    adcCfg.convMode     = ADC_CONV_MODE_ONESHOT;
    adcCfg.ctuMode      = ADC_CTU_MODE_TRIGGER;  /* Enable ADC triggering from BCTU and also normal chains - used for SW triggered conversion groups */
    adcCfg.overwriteEnable = true;

    /* Configure only the ADC instances which are used,
     * i.e. which have at least one channel assigned to at least one group. */
    uint16_t activeAdcMask = ADC_GetUsedAdcInstances(instance);
    for(idx = 0u; idx < FEATURE_BCTU_NUM_ADC; idx++)
    {
        if((activeAdcMask & (1u << idx)) != 0u)
        {
            ADC_DRV_Reset(idx);
            ADC_DRV_DoCalibration(idx);

            ADC_DRV_ConfigConverter(idx, &adcCfg);

            /* Used for SW triggered groups */
            ADC_DRV_EnableInterrupts(idx, ADC_FLAG_NORMAL_ENDCHAIN);
        }
    }

    /* Configure BCTU */
    bctuStatus = BCTU_DRV_Reset(instance, /* default timeout value 2ms*/ 2u);
    if(bctuStatus == STATUS_TIMEOUT)
    {
        status = STATUS_BUSY;
    }
    else
    {
        BCTU_DRV_GetDefaultConfig(&bctuCfg);
        bctuCfg.lowPowerModeEn   = true; /* Low power mode is enabled by default until a HW triggered group is enabled (only HW triggered groups use BCTU) */
        /* Used for HW triggered groups with a single conversion. By default enable new data interrupt for both ADCs */
        bctuCfg.newDataIntEnMask = (((uint8_t)1u << FEATURE_BCTU_NUM_ADC) - 1u);
        bctuCfg.listIntEn        = true;  /* Used for HW triggered groups with multiple conversions */

        BCTU_DRV_Config(instance, &bctuCfg);

        uint8_t bctuNumChansList = 0u; /* total number of channels programmed in BCTU list */

        /* Configure all HW triggered groups in BCTU.
         * SW triggered groups will be configured directly in ADC, each time ADC_StartGroupConversion() is called. */
        for(idx = 0u; idx < config->numGroups; idx++)
        {
            currentGroup = &(config->groupConfigArray[idx]);
            DEV_ASSERT(currentGroup->inputChannelArray != NULL);

            if(currentGroup->hwTriggerSupport == true)
            {
                status = ADC_ConfigHwTriggeredGroup(instance, currentGroup, bctuNumChansList);

                if(currentGroup->numChannels > 1u)
                {
                    bctuNumChansList += currentGroup->numChannels; /* Increment total number of channels programmed into BCTU LIST */
                }
            }
        }

#if defined (CUSTOM_DEVASSERT) || defined (DEV_ERROR_DETECT)
    {
        /* Check that a channel doesn't appear multiple times inside a group */
        for(idx = 0u; idx < config->numGroups; idx++)
        {
            uint8_t numOccurrences[ADC_CDR_COUNT]; /* total number of ADC channels is given by ADC_CDR_COUNT */
            uint8_t idx1;
            uint8_t chanIdx;
            currentGroup = &(config->groupConfigArray[idx]);

            /* First reset the number of occurrences of each channel */
            for(idx1 = 0u; idx1 < currentGroup->numChannels; idx1++)
            {
                chanIdx = ADC_GetChanIdx(currentGroup->inputChannelArray[idx1]);
                numOccurrences[chanIdx] = 0u;
            }
            /* Count the number of occurrences for each channel */
            for(idx1 = 0u; idx1 < currentGroup->numChannels; idx1++)
            {
                chanIdx = ADC_GetChanIdx(currentGroup->inputChannelArray[idx1]);
                numOccurrences[chanIdx]++;
            }
            for(idx1 = 0u; idx1 < currentGroup->numChannels; idx1++)
            {
                chanIdx = ADC_GetChanIdx(currentGroup->inputChannelArray[idx1]);
                if(numOccurrences[chanIdx] > 1u)
                {
                    DEV_ASSERT(false); /* A channel must not appear multiple times inside a group */
                }
            }
        }
    }
#endif /* defined (CUSTOM_DEVASSERT) || defined (DEV_ERROR_DETECT) */


        /* Move BCTU out of configuration mode */
        BCTU_DRV_WriteGlobalTriggerEn(instance, true);
    }
#endif /* defined(ADC_PAL_MPC574xC_G_R) */

    return status;
}


/*FUNCTION**********************************************************************
 *
 * Function Name : ADC_Deinit
 * Description   : This function resets the ADC PAL instance, including
 * the other platform specific HW units used together with ADC.
 *
 * Implements : ADC_Deinit_Activity
 *END**************************************************************************/
status_t ADC_Deinit(const adc_pal_instance_t instance)
{
    DEV_ASSERT(instance < NUMBER_OF_ADC_PAL_INSTANCES);

    adc_pal_state_t * const palState     = &(adcPalState[instance]);
    adc_group_state_t * const groupState = &(palState->activeGroupState);
    status_t status;

    if(palState->activeGroupFlag == true)
    {
        status = STATUS_BUSY;
    }
    else
    {
        /* Reset PAL state structure */
        palState->groupArray              = NULL;
        palState->numGroups               = 0u;
        groupState->bufferLength          = 0u;
        groupState->currentBufferOffset   = 0u;
        groupState->notificationEn        = false;

#if defined(ADC_PAL_S32K1xx)

        PDB_DRV_Deinit(instance);

        status = TRGMUX_DRV_SetTrigSourceForTargetModule(ADC_PAL_TRGMUX_IDX, TRGMUX_TRIG_SOURCE_DISABLED, adcPalTrgmuxTarget[instance]);
        DEV_ASSERT(status == STATUS_SUCCESS);

        ADC_DRV_Reset(instance);

        /* Disable interrupt from INT manager */
        IRQn_Type adcIrqId;
        adcIrqId = ADC_DRV_GetInterruptNumber(instance);
        INT_SYS_DisableIRQ(adcIrqId);

        status = STATUS_SUCCESS;

#elif defined(ADC_PAL_MPC574xC_G_R)

        /* Deinit BCTU and ADCs */
        status_t bctuStatus;
        uint8_t idx;
        bctuStatus = BCTU_DRV_Reset(instance, /* default timeout value 2ms*/ 2u);
        if(bctuStatus == STATUS_TIMEOUT)
        {
            status = STATUS_BUSY;
        }
        else
        {
            /* Deinit only the active ADC instances,
             * i.e. which have at least one channel assigned to at least one group. */
            uint16_t activeAdcMask = ADC_GetUsedAdcInstances(instance);
            for(idx = 0u; idx < FEATURE_BCTU_NUM_ADC; idx++)
            {
                if((activeAdcMask & (1u << idx)) != 0u)
                {
                    ADC_DRV_Reset(idx);
                }
            }

            status = STATUS_SUCCESS;
        }
#endif /* defined(ADC_PAL_MPC574xC_G_R) */
    }

    return status;
}

/*FUNCTION**********************************************************************
 *
 * Function Name : ADC_EnableHardwareTrigger
 * Description   : Enables the selected HW trigger for a conversion group,
 * if the conversion group has support for HW trigger.
 *
 * Implements : ADC_EnableHardwareTrigger_Activity
 *END**************************************************************************/
status_t ADC_EnableHardwareTrigger(const adc_pal_instance_t instance, const uint32_t groupIdx)
{
    DEV_ASSERT(instance < NUMBER_OF_ADC_PAL_INSTANCES);
    DEV_ASSERT(groupIdx < adcPalState[instance].numGroups);
    DEV_ASSERT(adcPalState[instance].groupArray != NULL);

    const adc_pal_state_t * const palState           = &(adcPalState[instance]);
    const adc_group_config_t * const currentGroupCfg = &(palState->groupArray[groupIdx]);
    status_t status       = STATUS_SUCCESS;
    status_t statusTrgmux = STATUS_SUCCESS;
    (void) statusTrgmux;

    DEV_ASSERT(currentGroupCfg->hwTriggerSupport == true);

    if(palState->activeGroupFlag == true)
    {
        /* A conversion group is already active */
        status = STATUS_BUSY;
    }
    else
    {
        /* Update ADC PAL and group state structures */
        ADC_PalStateUpdateStart(instance, groupIdx);

#if defined(ADC_PAL_S32K1xx)

        /* Enable the ADC interrupt from Interrupt Manager */
        IRQn_Type adcIrqId;
        adcIrqId = ADC_DRV_GetInterruptNumber(instance);
        INT_SYS_EnableIRQ(adcIrqId);

        /* The group shall be configured each time it is enabled. */
        ADC_ConfigGroup(instance, groupIdx, true);

        /* Enable in TRGMUX the selected HW trigger source for PDB.
         * Must be called after all PDB pre-triggers have been configured, to make sure no triggers occur during configuration. */
        statusTrgmux = TRGMUX_DRV_SetTrigSourceForTargetModule(ADC_PAL_TRGMUX_IDX, currentGroupCfg->triggerSource, adcPalTrgmuxTarget[instance]);
        DEV_ASSERT(statusTrgmux == STATUS_SUCCESS);

#elif defined(ADC_PAL_MPC574xC_G_R)
        const uint8_t numChans = (palState->groupArray[palState->activeGroupIdx]).numChannels;

        if(numChans == 1u)
        {
            INT_SYS_EnableIRQ(BCTU_ConvUpdate_IRQn);
        }
        else
        {
            INT_SYS_EnableIRQ(BCTU_ListLast_IRQn);
        }

        /* De-activate low-power mode. Any pending triggers will become active, immediately triggering a conversion. */
        BCTU_DRV_SetLowPowerMode(instance, false);

        /* All groups get configured at ADC PAL initialization,
         * so here they only need to be enabled. */
        BCTU_DRV_EnableHwTrigger(instance, currentGroupCfg->triggerSource);

#endif /* defined(ADC_PAL_MPC574xC_G_R) */

        status = STATUS_SUCCESS;
    }

    return status;
}


/*FUNCTION**********************************************************************
 *
 * Function Name : ADC_DisableHardwareTrigger
 * Description   : Disables the selected HW trigger for a conversion group,
 * if the conversion group has support for HW trigger.
 *
 * Implements : ADC_DisableHardwareTrigger_Activity
 *END**************************************************************************/
status_t ADC_DisableHardwareTrigger(const adc_pal_instance_t instance, const uint32_t groupIdx, const uint32_t timeout)
{
    DEV_ASSERT(instance < NUMBER_OF_ADC_PAL_INSTANCES);
    DEV_ASSERT(groupIdx < adcPalState[instance].numGroups);
    DEV_ASSERT(adcPalState[instance].groupArray != NULL);
    DEV_ASSERT(adcPalState[instance].groupArray[groupIdx].hwTriggerSupport == true);

    status_t status = STATUS_SUCCESS;

    if(adcPalState[instance].activeGroupIdx == groupIdx)
    {
#if defined(ADC_PAL_S32K1xx)
        /* Disable from TRGMUX the trigger source for PDB */
        status = TRGMUX_DRV_SetTrigSourceForTargetModule(ADC_PAL_TRGMUX_IDX, TRGMUX_TRIG_SOURCE_DISABLED, adcPalTrgmuxTarget[instance]);
        DEV_ASSERT(status == STATUS_SUCCESS);

        status = ADC_StopGroupBlocking(instance, timeout);

#elif defined(ADC_PAL_MPC574xC_G_R)
        const adc_pal_state_t * const palState   = &(adcPalState[instance]);
        const adc_trigger_source_t triggerSource = palState->groupArray[groupIdx].triggerSource;
        const uint8_t numChans                   = (palState->groupArray[palState->activeGroupIdx]).numChannels;
        uint32_t startTime, deltaTime;

        if(numChans == 1u)
        {
            INT_SYS_DisableIRQ(BCTU_ConvUpdate_IRQn);
        }
        else
        {
            INT_SYS_DisableIRQ(BCTU_ListLast_IRQn);
        }

        OSIF_TimeDelay(0u); /* Make sure OSIF timer is initialized. */

        startTime = OSIF_GetMilliseconds();
        deltaTime = 0u;

        /* Prevent new conversions from starting */
        BCTU_DRV_DisableHwTrigger(instance, triggerSource);

        /* Check if conversion is still running until execution finished or timeout occurred */
        while((BCTU_DRV_IsConvRunning(instance, triggerSource) == true) && (deltaTime < timeout))
        {
            deltaTime = OSIF_GetMilliseconds() - startTime;
        }

        if(deltaTime >= timeout)
        {
            status = STATUS_TIMEOUT;
        }
        else
        {
            status = STATUS_SUCCESS;

            /* Activate low-power mode */
            BCTU_DRV_SetLowPowerMode(instance, true);
        }

#endif /* defined(ADC_PAL_MPC574xC_G_R) */

        if(status == STATUS_SUCCESS)
        {
            adcPalState[instance].activeGroupFlag = false;
        }
    }

    return status;
}

/*FUNCTION**********************************************************************
 *
 * Function Name : ADC_StartGroupConversion
 * Description   : Starts the execution of a selected ADC conversion group.
 *
 * Implements : ADC_StartGroupConversion_Activity
 *END**************************************************************************/
status_t ADC_StartGroupConversion(const adc_pal_instance_t instance, const uint32_t groupIdx)
{
    DEV_ASSERT(instance < NUMBER_OF_ADC_PAL_INSTANCES);

    const adc_pal_state_t * const palState = &(adcPalState[instance]);

    DEV_ASSERT(groupIdx < palState->numGroups);
    DEV_ASSERT(palState->groupArray != NULL);
    DEV_ASSERT(palState->groupArray[groupIdx].hwTriggerSupport == false);

    status_t status;

    if(palState->activeGroupFlag == true)
    {
        /* A conversion group is already active */
        status = STATUS_BUSY;
    }
    else
    {
        /* Update ADC PAL and group state structures */
        ADC_PalStateUpdateStart(instance, groupIdx);

#if defined(ADC_PAL_S32K1xx)
        DEV_ASSERT(palState->groupArray[groupIdx].numChannels <= ADC_PAL_MAX_CONVS_IN_GROUP);

        bool hwTriggerEnabled = false;

        /* Enable the ADC interrupt from Interrupt Manager */
        IRQn_Type adcIrqId;
        adcIrqId = ADC_DRV_GetInterruptNumber(instance);
        INT_SYS_EnableIRQ(adcIrqId);

        ADC_ConfigGroup(instance, groupIdx, hwTriggerEnabled);

        /* Sw trigger PDB */
        PDB_DRV_SoftTriggerCmd(instance);

#elif defined(ADC_PAL_MPC574xC_G_R)

        const adc_group_config_t * currentGroupCfg  = &(palState->groupArray[groupIdx]);
        uint8_t idx, adcIdx;
        uint32_t currentChan;

        /* Assumes that all channels in the group target the same ADC instance */
        adcIdx = ADC_GetAdcIdx(currentGroupCfg->inputChannelArray[0u]);

        switch(adcIdx)
        {
        case 0u:
            INT_SYS_EnableIRQ(ADC0_EOC_IRQn);
            break;
        case 1u:
            INT_SYS_EnableIRQ(ADC1_EOC_IRQn);
            break;
        default:
            DEV_ASSERT(false);
            break;
        }

        for(idx = 0u; idx < currentGroupCfg->numChannels; idx++)
        {
            DEV_ASSERT(adcIdx == ADC_GetAdcIdx(currentGroupCfg->inputChannelArray[idx])); /* channels in the group must target the same ADC instance */
            currentChan = ADC_GetChanIdx(currentGroupCfg->inputChannelArray[idx]);

            ADC_DRV_EnableChannel(adcIdx, ADC_CONV_CHAIN_NORMAL, currentChan);
        }

        ADC_DRV_StartConversion(adcIdx, ADC_CONV_CHAIN_NORMAL);

#endif /* defined(ADC_PAL_MPC574xC_G_R) */

        status = STATUS_SUCCESS;
    }

    return status;
}

/*FUNCTION**********************************************************************
 *
 * Function Name : ADC_StopGroupConversion
 * Description   : Stops the execution of the ADC group currently executing.
 *
 * Implements : ADC_StopGroupConversion_Activity
 *END**************************************************************************/
status_t ADC_StopGroupConversion(const adc_pal_instance_t instance, const uint32_t groupIdx, const uint32_t timeout)
{
    DEV_ASSERT(instance < NUMBER_OF_ADC_PAL_INSTANCES);
    DEV_ASSERT(groupIdx < adcPalState[instance].numGroups);
    DEV_ASSERT(adcPalState[instance].groupArray != NULL);
    DEV_ASSERT(adcPalState[instance].groupArray[groupIdx].hwTriggerSupport == false);

    status_t status = STATUS_SUCCESS;

    if((adcPalState[instance].activeGroupIdx == groupIdx) && (adcPalState[instance].activeGroupFlag == true))
    {
#if defined(ADC_PAL_S32K1xx)

        /* Disable the ADC interrupt from Interrupt Manager */
        IRQn_Type adcIrqId;
        adcIrqId = ADC_DRV_GetInterruptNumber(instance);
        INT_SYS_DisableIRQ(adcIrqId);

        status = ADC_StopGroupBlocking(instance, timeout);

#elif defined(ADC_PAL_MPC574xC_G_R)

        const adc_group_config_t * currentGroupCfg  = &(adcPalState[instance].groupArray[groupIdx]);
        uint8_t idx, adcIdx;
        uint32_t currentChan;
        uint32_t startTime, deltaTime;

        OSIF_TimeDelay(0u); /* Make sure OSIF timer is initialized. */

        startTime = OSIF_GetMilliseconds();
        deltaTime = 0u;

        /* Assumes that all channels in the group target the same ADC */
        adcIdx = ADC_GetAdcIdx(currentGroupCfg->inputChannelArray[0u]);

        switch(adcIdx)
        {
        case 0u:
            INT_SYS_DisableIRQ(ADC0_EOC_IRQn);
            break;
        case 1u:
            INT_SYS_DisableIRQ(ADC1_EOC_IRQn);
            break;
        default:
            DEV_ASSERT(false);
            break;
        }

        /* Disable channels from current conversion group */
        for(idx = 0u; idx < currentGroupCfg->numChannels; idx++)
        {
            currentChan = ADC_GetChanIdx(currentGroupCfg->inputChannelArray[idx]);

            ADC_DRV_DisableChannel(adcIdx, ADC_CONV_CHAIN_NORMAL, currentChan);
        }

        /* Check if conversion is still running, until execution finished or timeout occurred */
        while(((ADC_DRV_GetStatusFlags(adcIdx) & ADC_FLAG_NORMAL_STARTED) != 0u) && (deltaTime < timeout))
        {
            deltaTime = OSIF_GetMilliseconds() - startTime;
        }

        /* Clear status & pending flags corresponding to a conversion group
         * which may have finished after interrupts have been disabled */
        ADC_DRV_ClearStatusFlags(adcIdx, ADC_FLAG_NORMAL_ENDCHAIN);
        ADC_Type * const s_adcBase[ADC_INSTANCE_COUNT] = ADC_BASE_PTRS;
        ADC_Type * const base = s_adcBase[instance];
        for(idx = 0u; idx < ADC_CEOCFR_COUNT; idx++)
        {
            base->CEOCFR[idx] |= 0xFFFFFFFFu;
        }

        if(deltaTime >= timeout)
        {
            status = STATUS_TIMEOUT;
        }
        else
        {
            status = STATUS_SUCCESS;
        }
#endif /* defined(ADC_PAL_MPC574xC_G_R) */

        if(status == STATUS_SUCCESS)
        {
            adcPalState[instance].activeGroupFlag = false;
        }
    }
    else
    {
        status = STATUS_ERROR;
    }

    return status;
}


/*FUNCTION**********************************************************************
 *
 * Function Name : ADC_EnableNotification
 * Description   : Enables the notification callback for a configured group.
 *
 * Implements : ADC_EnableNotification_Activity
 *END**************************************************************************/
status_t ADC_EnableNotification(const adc_pal_instance_t instance, const uint32_t groupIdx)
{
    DEV_ASSERT(instance < NUMBER_OF_ADC_PAL_INSTANCES);

    adc_pal_state_t * const state = &(adcPalState[instance]);

    DEV_ASSERT(groupIdx < state->numGroups);

    status_t status = STATUS_SUCCESS;

    if((groupIdx != state->activeGroupIdx) || (state->activeGroupFlag == false))
    {
        status = STATUS_ERROR;
    }
    else
    {
#if (defined(ADC_PAL_S32K1xx) || defined(ADC_PAL_MPC574xC_G_R))

        DEV_ASSERT(state->groupArray[state->activeGroupIdx].callback != NULL);

        state->activeGroupState.notificationEn = true;

#endif /* (defined(ADC_PAL_S32K1xx) || defined(ADC_PAL_MPC574xC_G_R)) */

    }

    return status;
}

/*FUNCTION**********************************************************************
 *
 * Function Name : ADC_DisableNotification
 * Description   : Disables the notification callback for a configured group.
 *
 * Implements : ADC_DisableNotification_Activity
 *END**************************************************************************/
status_t ADC_DisableNotification(const adc_pal_instance_t instance, const uint32_t groupIdx)
{
    DEV_ASSERT(instance < NUMBER_OF_ADC_PAL_INSTANCES);

    adc_pal_state_t * const state = &(adcPalState[instance]);

    DEV_ASSERT(groupIdx < state->numGroups);

    status_t status = STATUS_SUCCESS;

    if((groupIdx != state->activeGroupIdx) || (state->activeGroupFlag == false))
    {
        status = STATUS_ERROR;
    }
    else
    {
#if (defined(ADC_PAL_S32K1xx) || defined(ADC_PAL_MPC574xC_G_R))

        state->activeGroupState.notificationEn = false;

#endif /* (defined(ADC_PAL_S32K1xx) || defined(ADC_PAL_MPC574xC_G_R)) */
    }

    return status;
}


/*******************************************************************************
 * Private Functions
 ******************************************************************************/

#if defined(ADC_PAL_S32K1xx)

/*FUNCTION**********************************************************************
 *
 * Function Name : ADC_S32K1xx_IrqHandler
 * Description   : Implements interrupt handler functionality to be called from
 * interrupt handlers for each ADC.
 *
 *END**************************************************************************/
void ADC_S32K1xx_IrqHandler(const adc_pal_instance_t instance)
{
    adc_pal_state_t * const palState                = &(adcPalState[instance]);
    adc_group_state_t * const groupState            = &(palState->activeGroupState);
    const adc_group_config_t * const activeGroupCfg = &(palState->groupArray[palState->activeGroupIdx]);
    uint8_t controlChanIdx = 0u;

    uint16_t * result = &(activeGroupCfg->resultBuffer[groupState->currentBufferOffset]);

    /* Read all conversion results */
    for(controlChanIdx = 0u; controlChanIdx < activeGroupCfg->numChannels; controlChanIdx++)
    {
        ADC_DRV_GetChanResult(instance, controlChanIdx, result);    /* interrupt flag is cleared when reading the result */
        result++;
    }
    /* Increment offset in result buffer */
    groupState->currentBufferOffset = (uint16_t)((groupState->currentBufferOffset + activeGroupCfg->numChannels) % groupState->bufferLength);


    if(activeGroupCfg->hwTriggerSupport == false) /* Continuous mode currently supported only for SW triggered groups */
    {
        if(activeGroupCfg->continuousConvEn == true)
        {
            /* Sw trigger PDB */
            PDB_DRV_SoftTriggerCmd(instance);
        }
        else
        {
            palState->activeGroupFlag = false;
        }
    }

    /* Call notification callback, if it is enabled */
    ADC_CallNotificationCb(palState);
}


/*FUNCTION**********************************************************************
 *
 * Function Name : ADC_ConfigPdbAndPretriggers
 * Description   : Configures PDB and PDB pretriggers
 *
 *END**************************************************************************/
static inline void ADC_ConfigPdbAndPretriggers(const uint32_t instance, const pdb_trigger_src_t trgSrc, const adc_group_config_t * currentGroupCfg)
{
    pdb_timer_config_t pdbCfg;
    pdb_adc_pretrigger_config_t pdbPretrigCfg;
    uint8_t idx;

    /* PDB driver instance configuration */
    pdbCfg.loadValueMode        = PDB_LOAD_VAL_IMMEDIATELY;
    pdbCfg.seqErrIntEnable      = false;
    pdbCfg.clkPreDiv            = PDB_CLK_PREDIV_BY_1;
    pdbCfg.clkPreMultFactor     = PDB_CLK_PREMULT_FACT_AS_1;
    pdbCfg.dmaEnable            = false;
    pdbCfg.intEnable            = false;
    pdbCfg.continuousModeEnable = false; /* Continuous mode refers to Counter being reset at zero - not used in ADC PAL */

    pdbCfg.triggerInput         = trgSrc;
    PDB_DRV_Init(instance, &pdbCfg);

    /* PDB pre-triggers configuration */
    pdbPretrigCfg.preTriggerEnable           = true;
    pdbPretrigCfg.preTriggerOutputEnable     = false;

    pdbPretrigCfg.preTriggerBackToBackEnable = false; /* the first pretrigger in the group must not have BB enabled */
    pdbPretrigCfg.adcPreTriggerIdx           = 0u;
    PDB_DRV_ConfigAdcPreTrigger(instance, ADC_PAL_PDB_CHAN, &pdbPretrigCfg);

    pdbPretrigCfg.preTriggerBackToBackEnable = true; /* the rest of pretriggers in the group must have BB enabled */
    for(idx = 1u; idx < currentGroupCfg->numChannels; idx++)
    {
        pdbPretrigCfg.adcPreTriggerIdx = idx;
        PDB_DRV_ConfigAdcPreTrigger(instance, ADC_PAL_PDB_CHAN, &pdbPretrigCfg);
    }

    PDB_DRV_Enable(instance);
}


/*FUNCTION**********************************************************************
 *
 * Function Name : ADC_ConfigGroup
 * Description   : Configures ADC input channels and PDB for a conversion group
 *
 *END**************************************************************************/
static void ADC_ConfigGroup(const uint32_t instance, const uint32_t groupIdx, const bool hwTriggerFlag)
{
    const adc_group_config_t * const currentGroupCfg  = &(adcPalState[instance].groupArray[groupIdx]);

    pdb_trigger_src_t pdbTrigSrc;
    if(hwTriggerFlag == true)
    {
        pdbTrigSrc = PDB_TRIGGER_IN0;
    }
    else
    {
        pdbTrigSrc = PDB_SOFTWARE_TRIGGER;
    }

    /* Configure PDB instance and pre-triggers */
    ADC_ConfigPdbAndPretriggers(instance, pdbTrigSrc, currentGroupCfg);

    /* Configure ADC channels */
    adc_chan_config_t adcChanCfg;
    uint8_t idx;

    adcChanCfg.interruptEnable = false; /* interrupt is disabled for all conversion, except the last one in the group */
    for(idx = 0u; idx < currentGroupCfg->numChannels - 1u; idx++)
    {
        adcChanCfg.channel = currentGroupCfg->inputChannelArray[idx]; /* set the ADC input channel */

        ADC_DRV_ConfigChan(instance, idx, &adcChanCfg); /* conversion complete flag is cleared implicitly when writing a new configuration */
    }
    adcChanCfg.interruptEnable = true; /* enable interrupt for last conversion in the group */
    adcChanCfg.channel         = currentGroupCfg->inputChannelArray[idx]; /* set the ADC input channel */
    ADC_DRV_ConfigChan(instance, idx, &adcChanCfg); /* configure the last conversion in the group */
}

/*FUNCTION**********************************************************************
 *
 * Function Name : ADC_StopGroupBlocking
 * Description   : Stops a group of conversions within a given timeout interval
 *
 *END**************************************************************************/
static status_t ADC_StopGroupBlocking(const uint32_t instance, const uint32_t timeout)
{
    const adc_pal_state_t * const palState = &(adcPalState[instance]);
    uint32_t startTime, deltaTime;
    status_t status;
    pdb_adc_pretrigger_config_t pdbPretrigCfg;
    uint8_t idx;

    /* Make sure OSIF timer is initialized. */
    OSIF_TimeDelay(0u);

    startTime = OSIF_GetMilliseconds();
    deltaTime = 0u;

    /* Reset PDB pre-trigger configurations to stop PDB from triggering other conversions in the group */
    pdbPretrigCfg.preTriggerEnable           = false;
    pdbPretrigCfg.preTriggerOutputEnable     = false;
    pdbPretrigCfg.preTriggerBackToBackEnable = false;
    for(idx = 0u; idx < palState->groupArray[palState->activeGroupIdx].numChannels; idx++)
    {
        pdbPretrigCfg.adcPreTriggerIdx = idx;
        PDB_DRV_ConfigAdcPreTrigger(instance, ADC_PAL_PDB_CHAN, &pdbPretrigCfg);
    }
    /* Completely stop PDB */
    PDB_DRV_Deinit((uint32_t)instance);

    /* Wait for current ADC active conversion to finish execution */
    ADC_Type * const adcBase[ADC_INSTANCE_COUNT] = ADC_BASE_PTRS;
    const ADC_Type * const base                  = adcBase[instance];
    while((ADC_GetConvActiveFlag(base) == true) && (deltaTime < timeout))
    {
        deltaTime = OSIF_GetMilliseconds() - startTime;
    }

    if(deltaTime >= timeout)
    {
        status = STATUS_TIMEOUT;
    }
    else
    {
        status = STATUS_SUCCESS;
    }

    return status;
}

#elif defined(ADC_PAL_MPC574xC_G_R)

/*FUNCTION**********************************************************************
 *
 * Function Name : ADC_GetChanIdx
 * Description   : Returns ADC channel index, demapped from an ADC PAL input channel
 *
 *END**************************************************************************/
static inline uint8_t ADC_GetChanIdx(const adc_input_chan_t adcPalInputChan)
{
    return (uint8_t)(adcPalInputChan & ADC_CHAN_CHAN_IDX_MASK);
}

/*FUNCTION**********************************************************************
 *
 * Function Name : ADC_GetAdcIdx
 * Description   : Returns ADC index, demapped from an ADC PAL input channel
 *
 *END**************************************************************************/
static inline uint8_t ADC_GetAdcIdx(const adc_input_chan_t adcPalInputChan)
{
    return (uint8_t)((adcPalInputChan & ADC_CHAN_ADC_IDX_MASK) >> 12u);
}


/*FUNCTION**********************************************************************
 *
 * Function Name : ADC_GetUsedAdcMask
 * Description   : Returns a bitmask with each bit (starting from LSB) corresponding to
 * an ADC instance. If the corresponding bit is set to "1" the ADC instance is targeted
 * by at least one conversion within a group.
 *
 *END**************************************************************************/
static uint16_t ADC_GetUsedAdcInstances(const adc_pal_instance_t instance)
{
    adc_pal_state_t * const palState = &(adcPalState[instance]);
    const adc_group_config_t * currentGroup;
    uint16_t usedAdcMask = 0u;
    uint8_t idx;

    for(idx = 0u; idx < palState->numGroups; idx++)
    {
        currentGroup = &(palState->groupArray[idx]);
        /* Assumes that a group contains channels targeting the same ADC index.
         * This assumption is checked with DEV_ASSERT in other part of the code. */
        usedAdcMask |= ((uint16_t)1u << ADC_GetAdcIdx(currentGroup->inputChannelArray[0u]));
    }

    return usedAdcMask;
}

/*FUNCTION**********************************************************************
 *
 * Function Name : ADC_MPC574xC_G_R_HwTrigIrqHandler
 * Description   : Implements interrupt handler functionality for HW triggered
 * conversion groups - to be called from BCTU interrupt handler.
 *
 *END**************************************************************************/
void ADC_MPC574xC_G_R_HwTrigIrqHandler(const adc_pal_instance_t instance, const uint8_t adcIdx)
{
    adc_pal_state_t * const palState                = &(adcPalState[instance]);
    adc_group_state_t * const groupState            = &(palState->activeGroupState);
    const adc_group_config_t * const activeGroupCfg = &(palState->groupArray[palState->activeGroupIdx]);

    uint16_t * result = &(activeGroupCfg->resultBuffer[groupState->currentBufferOffset]);

    if(activeGroupCfg->numChannels == 1u)
    {
        *result = BCTU_DRV_GetConvData(instance, adcIdx);

        /* Increment offset in result buffer */
        groupState->currentBufferOffset = (groupState->currentBufferOffset + 1u) % groupState->bufferLength;

        BCTU_DRV_ClearStatusFlag(instance, adcIdx, BCTU_FLAG_NEW_DATA_AVAILABLE);
        DEV_ASSERT(BCTU_DRV_GetStatusFlag(instance, adcIdx, BCTU_FLAG_DATA_OVERRUN) == false);
    }
    else
    {
        /* Read all conversion results for the current conversion group */
        ADC_DRV_GetConvResultsToArray(adcIdx, ADC_CONV_CHAIN_CTU, result, activeGroupCfg->numChannels);

        /* Increment offset in result buffer */
        groupState->currentBufferOffset = (groupState->currentBufferOffset + activeGroupCfg->numChannels) % groupState->bufferLength;

        BCTU_DRV_ClearStatusFlag(instance, adcIdx, BCTU_FLAG_LIST_LAST_CONV);
        /* Clear because conversion results are not read from BCTU result register */
        BCTU_DRV_ClearStatusFlag(instance, adcIdx, BCTU_FLAG_NEW_DATA_AVAILABLE);
        /* Clear because BCTU result register is overwritten */
        BCTU_DRV_ClearStatusFlag(instance, adcIdx, BCTU_FLAG_DATA_OVERRUN);
    }

    /* Call notification callback, if it is enabled */
    ADC_CallNotificationCb(palState);
}


/*FUNCTION**********************************************************************
 *
 * Function Name : ADC_MPC574xC_G_R_SwTrigIrqHandler
 * Description   : Implements interrupt handler functionality for SW triggered
 * conversion groups - to be called from ADC End of Chain (ECH) interrupt handler.
 *
 *END**************************************************************************/
void ADC_MPC574xC_G_R_SwTrigIrqHandler(const adc_pal_instance_t instance,  const uint8_t adcIdx)
{
    adc_pal_state_t * const palState                = &(adcPalState[instance]);
    adc_group_state_t * const groupState            = &(palState->activeGroupState);
    const adc_group_config_t * const activeGroupCfg = &(palState->groupArray[palState->activeGroupIdx]);

    uint16_t * result = &(activeGroupCfg->resultBuffer[groupState->currentBufferOffset]);

    DEV_ASSERT(activeGroupCfg->hwTriggerSupport == false); /* This is only used for SW triggered conversion groups */

    /* Read all conversion results for the current conversion group */
    ADC_DRV_GetConvResultsToArray(adcIdx, ADC_CONV_CHAIN_NORMAL, result, activeGroupCfg->numChannels);

    /* Increment offset in result buffer */
    groupState->currentBufferOffset = (uint16_t)((groupState->currentBufferOffset + activeGroupCfg->numChannels) % groupState->bufferLength);

    ADC_DRV_ClearStatusFlags(adcIdx, ADC_FLAG_NORMAL_ENDCHAIN);

    /* Call notification callback, if it is enabled */
    ADC_CallNotificationCb(palState);

    if(activeGroupCfg->continuousConvEn == true) /* Continuous mode currently supported only for SW triggered groups */
    {
        ADC_DRV_StartConversion(adcIdx, ADC_CONV_CHAIN_NORMAL);
    }
    else
    {
        palState->activeGroupFlag = false;
    }
}

/*FUNCTION**********************************************************************
 *
 * Function Name : ADC_ConfigHwTriggeredGroup
 * Description   : Configure BCTU for a ADC PAL HW triggered group
 *
 *END**************************************************************************/
static status_t ADC_ConfigHwTriggeredGroup(const adc_pal_instance_t instance, const adc_group_config_t * const groupConfig, uint8_t bctuStartListIdx)
{
    const adc_input_chan_t * const inputChans = groupConfig->inputChannelArray;
    bctu_trig_config_t bctuTrigConfig;
    status_t bctuStatus;
    status_t status          = STATUS_SUCCESS;
    const uint8_t numChans   = groupConfig->numChannels;

    DEV_ASSERT(numChans > 0u);

    bctuTrigConfig.adcTargetMask = (uint8_t)1u << ADC_GetAdcIdx(inputChans[0]);
    bctuTrigConfig.hwTriggersEn  = groupConfig->hwTriggerSupport;
    bctuTrigConfig.loopEn        = groupConfig->continuousConvEn;

    if(numChans == 1u) /* Configure groups containing a single conversion. */
    {
        bctuStatus = BCTU_DRV_ConfigTriggerSingle(instance, groupConfig->triggerSource, &bctuTrigConfig, ADC_GetChanIdx(inputChans[0]));
        DEV_ASSERT(bctuStatus == STATUS_SUCCESS);
    }
    else /* Configure groups containing more than a single conversion. */
    {
        /* Prepare array with adc channel indexes expressed in normal format, accepted by the BCTU driver */
        uint8_t adcChanIdx[ADC_PAL_MAX_CONVS_IN_GROUP_HW_TRIG];
        uint8_t idx;
        for(idx = 0u; idx < numChans; idx++)
        {
            adcChanIdx[idx] = ADC_GetChanIdx(inputChans[idx]);

            /* Channels in the group must target the same ADC instance */
            DEV_ASSERT(ADC_GetAdcIdx(inputChans[0u]) == ADC_GetAdcIdx(inputChans[idx]));
        }

        /* For groups which include a minimum of 2 conversions, the total number of conversions within all these groups shall be less than
         * or equal with the number of BCTU LIST HW registers. Otherwise the first groups will be overwritten. */
        if((bctuStartListIdx + numChans) >= ADC_PAL_MAX_CONVS_IN_GROUP_HW_TRIG)
        {
            status = STATUS_ERROR;
        }
        else
        {
            bctuStatus = BCTU_DRV_ConfigTriggerList(instance, groupConfig->triggerSource, &bctuTrigConfig, bctuStartListIdx);
            DEV_ASSERT(bctuStatus == STATUS_SUCCESS);

            BCTU_DRV_SetConvListArray(instance, bctuStartListIdx, adcChanIdx, numChans);
        }
    }

    return status;
}

#endif /* defined(ADC_PAL_MPC574xC_G_R) */


#if (defined(ADC_PAL_S32K1xx) || defined(ADC_PAL_MPC574xC_G_R))


/*FUNCTION**********************************************************************
 *
 * Function Name : ADC_PalStateUpdateStart
 * Description   : Update PAL state structure with changes for StartGroupConversion and
 * EnableHardwareTigger.
 *
 *END**************************************************************************/
static void ADC_PalStateUpdateStart(const adc_pal_instance_t instance, const uint32_t groupIdx)
{
    adc_pal_state_t * palState                       = &(adcPalState[instance]);
    adc_group_state_t * groupState                   = &(palState->activeGroupState);
    const adc_group_config_t * const currentGroupCfg = &(palState->groupArray[groupIdx]);

    palState->activeGroupFlag       = true;
    palState->activeGroupIdx        = groupIdx;
    groupState->currentBufferOffset = 0u;
    groupState->bufferLength        = (uint16_t)(currentGroupCfg->numChannels * currentGroupCfg->numSetsResultBuffer);
    if(currentGroupCfg->callback != NULL)
    {
        groupState->notificationEn = true; /* enable notification by default if callback is available */
    }
}


/*FUNCTION**********************************************************************
 *
 * Function Name : ADC_CallNotificationCb
 * Description   : Calls the notification callback for the current active group,
 * if it is enabled.
 *
 *END**************************************************************************/
static void ADC_CallNotificationCb(const adc_pal_state_t * const palState)
{
    const adc_group_state_t * const groupState      = &(palState->activeGroupState);
    const adc_group_config_t * const activeGroupCfg = &(palState->groupArray[palState->activeGroupIdx]);

    /* Call notification callback, if it is enabled */
    if(groupState->notificationEn)
    {
        adc_callback_info_t cbInfo;
        cbInfo.groupIndex = palState->activeGroupIdx;

        if(groupState->currentBufferOffset == 0u)
        {
            cbInfo.resultBufferTail = (uint16_t)(groupState->bufferLength - 1u); /* set tail to the last position in buffer */
        }
        else
        {
            cbInfo.resultBufferTail = (uint16_t)(groupState->currentBufferOffset - 1u); /* set tail to last written position  */
        }

        (*(activeGroupCfg->callback))(&cbInfo, activeGroupCfg->callbackUserData);
    }
}


#endif /* (defined(ADC_PAL_S32K1xx) || defined(ADC_PAL_MPC574xC_G_R)) */

/*******************************************************************************
 * EOF
 ******************************************************************************/
